'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import data

import sys
import pygame
import random 

WIDTH = 800
HEIGHT = 600
MAX_STARS = 50
STAR_SPEED = 2

PLAYER_WIDTH = 100
PLAYER_HEIGHT = 63


#sound
pygame.mixer.init()
global bullet_sound
bullet_sound = pygame.mixer.Sound(data.load("39458__the-bizniss__laser-4.wav"))
global bg_sound
bg_sound = pygame.mixer.Sound(data.load("Rolemusic_Yellow_Dust.ogg"))
global enemy_burst
enemy_dead_sound = pygame.mixer.Sound(data.load("51464__smcameron__bombexplosion.wav"))
global mutagen_up_sound
mutagen_up_sound = pygame.mixer.Sound(data.load("30342__junggle__waterdrop25.wav"))
global player_damage_sound
player_damage_sound = pygame.mixer.Sound(data.load("33215__hardpcm__chip075.wav"))
global player_dead_sound
player_dead_sound = pygame.mixer.Sound(data.load("32954__hardpcm__chip054.wav"))

class SpriteBase(pygame.sprite.Sprite):
	def __init__(self, imagePath):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.image.load( data.load(imagePath) ) #.convert()
                self.image_orig = self.image
		self.rect = self.image.get_rect()
	
		self.explosion_images = self.load_sliced_sprites(64, 60, 'explosion.png')
		self.dying = False
		self.life = 3
		self.dying_count = 0
                self.damage_image = pygame.image.load( data.load('enemy-red.png') )
                self.damage_count = 0
                self.damage = False

	def load_sliced_sprites(self, w, h, filename):
		images = []
		master_image = pygame.image.load( data.load( filename ) )
		master_width, master_height = master_image.get_size()
		for i in xrange(int(master_width/w)):
          		images.append(master_image.subsurface((i*w,0,w,h)))
		images.reverse()		
		return images

	def register_damage(self, damage):
                self.life -= damage
                self.damage = True
	        if self.life <= 0:
		        self.dying = True

	def do_dying(self): 
		if self.dying_count % 2 == 0 :
			try:
				self.image = self.explosion_images.pop()
			except: 
				self.kill()
		self.dying_count += 1	
                
		
class Player(SpriteBase):	
	def __init__(self):
		SpriteBase.__init__(self, 'pod.png')
		self.life = 50
                self.mutagen = 0
                self.mutated = False
                self.mutatedCount = 0
		
	def update(self):
                if self.mutated:
                        self.image = pygame.image.load( data.load('ship.png') )
                        self.mutatedCount += 1
                        if self.mutatedCount > 300:
                                self.mutated = False
                                self.mutatedCount = 0
                        self.rect = self.image.get_rect()
                        
                else:        
                        self.image = pygame.image.load( data.load('pod.png') )	
           		self.rect = self.image.get_rect()
		if self.dying:
			self.do_dying()
                player_pos = [0, 0]
		mouse_pos = pygame.mouse.get_pos()
                #Height
                if mouse_pos[1] < (PLAYER_HEIGHT/2):
                        player_pos[1] = (PLAYER_HEIGHT/2)
                elif mouse_pos[1] > (HEIGHT - PLAYER_HEIGHT/2):
                        player_pos[1] = (HEIGHT - PLAYER_HEIGHT/2)
                else:
                        player_pos[1] = mouse_pos[1]
                #Width
                #if mouse_pos[0] < (PLAYER_WIDTH):
                #        player_pos[0] = (PLAYER_WIDTH*2)
                #elif mouse_pos[0] > (WIDTH - PLAYER_WIDTH):
                #        player_pos[0] = (WIDTH - PLAYER_WIDTH*2)
                #else:
                player_pos[0] = mouse_pos[0]
                player_pos[0] = mouse_pos[0]
                self.rect.center = (player_pos[0], player_pos[1])
        
        def mutagen_up(self):
                self.mutagen += 1


class Bullett(SpriteBase):
	def __init__(self, imageFile = 'bullet.png'):
		SpriteBase.__init__(self, imageFile)
		#bullet from front of player
		self.mouse_pos = pygame.mouse.get_pos()
		self.rect.center = ( (self.mouse_pos[0] + PLAYER_WIDTH / 2), (self.mouse_pos[1]) )
                self.damage = 1

	def update(self):
		self.rect = self.rect.move(10,0)

class Torpedo(Bullett):
        def __init__(self):
                Bullett.__init__(self, 'torpedo.png')
        	self.rect.center = ( (self.mouse_pos[0] + PLAYER_WIDTH / 2), (self.mouse_pos[1] - (97/2)) )
                self.damage = 3


class Enemy(SpriteBase):
	def __init__(self):
		SpriteBase.__init__(self, 'enemy.png')
		s_width = WIDTH - 100
		s_height = random.randrange(start = 20, stop = HEIGHT - 20, step = self.rect.height)
		self.rect.center = (s_width, s_height)
		self.movementtype = random.randint(1,3)
		self.xmovement = -4

	def update(self):
                if self.damage == True:
                        self.image = self.damage_image
                        self.damage_count += 1
                        if self.damage_count > 5:
                                self.image = self.image_orig
                                self.damage = False
                                self.damage_count = 0
		if self.dying:
			self.do_dying()
                        enemy_dead_sound.play()
		elif self.movementtype == 1:
			self.rect = self.rect.move(self.xmovement,0)
		elif self.movementtype == 2:
			self.rect = self.rect.move(self.xmovement,random.randint(-2,0))
		elif self.movementtype == 3:
			self.rect = self.rect.move(self.xmovement,random.randint(0,2))
       

class Star(pygame.sprite.Sprite):
	def __init__(self):
		self._layer = -1
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.image.load( data.load('star.png')) #.convert()
		self.rect = self.image.get_rect()
		self.start_over()

	def start_over(self):
		x_start = random.randrange( start = 0, stop = WIDTH - 1 )
		y_start = random.randrange( start = 0, stop = HEIGHT - 1)
		self.rect.center = (x_start, y_start)
		
	def update(self):
		self.rect = self.rect.move(-STAR_SPEED, 0)
		if self.rect.left < 0:
			self.start_over()

class Board(pygame.sprite.Sprite):
        def __init__(self, image_file):
                pygame.sprite.Sprite.__init__(self)
                self.image = pygame.image.load( data.load( image_file ) )
                self.rect = self.image.get_rect()
                self.rect.center = (400, 300)

class Mutagen(SpriteBase):
	def __init__(self, x_y):
		SpriteBase.__init__(self, 'goup.png')
		self.rect.center = (x_y[0], x_y[1])
                self.display_count = 0

	def update(self):
                rand = random.randrange( start=-1, stop=2 , step=1)
		self.rect = self.rect.move(-1, random.randrange( start=-1, stop=2 , step=1))

class LifeBar(SpriteBase):	
	def __init__(self):
                SpriteBase.__init__(self, 'goup.png')
		self.lifebarImages = self.load_sliced_sprites(108, 18, 'lifebar.png')
                self.image = self.lifebarImages[len(self.lifebarImages)-1]
                self.rect = self.image.get_rect()
		self.rect.center = (80, 30)

        def update_damage(self, pHealth):
                self.image = self.lifebarImages[pHealth]

class MutaBar(SpriteBase):	
	def __init__(self):
                SpriteBase.__init__(self, 'goup.png')
		self.mutabarImages = self.load_sliced_sprites(102, 12, 'mutabar.png')
                self.mutabarImages.reverse()
                self.image = self.mutabarImages[0]#len(self.mutabarImages)]
                self.rect = self.image.get_rect()
		self.rect.center = (80, 50)

        def update_mBar(self, mGen):
                self.image = self.mutabarImages[mGen]



class Label(pygame.sprite.Sprite):
        def __init__(self, text=""): 
                pygame.sprite.Sprite.__init__(self)       
                self.font = pygame.font.SysFont("Arial MS", 30) 
                self.text = " "
                self.center = (100, 100)
                self.display_count = 0

        def update(self):
                if self.display_count > 30:
                        self.display_count = 0
                        self.text = ""
                self.image = self.font.render( self.text, True, (255, 140, 0) )
                self.rect = self.image.get_rect()
                self.rect.center = self.center
                self.display_count += 1

class ScoreLabel(Label):
       def __init(self, text=""):
               Label.__init__(self, text)

       def update(self):
               self.image = self.font.render( self.text, True, (135, 206, 250) )
               self.rect = self.image.get_rect()
               self.rect.center = self.center                


def init_stars(sprite_collection):
	for i in range(MAX_STARS):
		newStar = Star()
		sprite_collection.add(newStar)
	return sprite_collection

def spawn_mutagen(x_y, groups):
        rand = random.randrange( start=-5, stop=5, step=1)
        if rand == 0:
                newMutagen = Mutagen(x_y)
                for group in groups:
                       group.add(newMutagen)
                return True
        return False

        
def main():
	print "Hello from your game's main()"
	print data.load('sample.txt').read()

        def kill_out_of_bounds_sprites():
                def check_sprite(sprite):
                        center = sprite.rect.center
                        if center[0] < 0 or center[0] > WIDTH or center[1] < 0 or center[1] > HEIGHT:
                                sprite.kill()
                for sprite in enemySprites:
                        check_sprite(sprite)
                for sprite in bullettSprites:
                        check_sprite(sprite)

        
	pygame.init()

	pygame.display.set_caption("PyFlux")
	
	size = width, height = WIDTH, HEIGHT
	screen = pygame.display.set_mode(size)

	background = pygame.Surface(screen.get_size())
	background.fill((0, 0, 0))
	screen.blit(background, (0, 0),  )

        startG = True
        quitG = False

        end_board = Board('gameover.png')
        start_board = Board('menu.png')

        bSprites = pygame.sprite.Group(start_board)
        bSprites.update()
        bSprites.draw(screen)        
	pygame.display.flip()
        while startG:
                for event in pygame.event.get():
                        if event.type == pygame.QUIT:
                                return
                        elif event.type == pygame.MOUSEBUTTONDOWN:
                                startG = False
                        
                               
        bSprites.clear(screen, background) 
        bSprites.add(end_board)
	bSprites.remove(start_board)

	player = Player()
	lifebar = LifeBar()
	mutabar = MutaBar()
	allSprites = pygame.sprite.Group(player)
	starSprites = init_stars(pygame.sprite.LayeredUpdates())
	allSprites.add(lifebar)
	allSprites.add(mutabar)
	allSprites.add(starSprites)
	enemySprites = pygame.sprite.Group()
	bullettSprites = pygame.sprite.Group()
        mutagenSprites = pygame.sprite.Group()
        mBarCount = 0
        score = 0
        exmove = -4
       

        label = Label()
        scoreLabel = ScoreLabel()
        allSprites.add(label)
        allSprites.add(scoreLabel)

        bg_sound.set_volume(0.5)
        bg_sound.play()

        scoreLabel.text = str(score)
        scoreLabel.center = (600, 30)
        
	pygame.mouse.set_visible(False)
	clock = pygame.time.Clock()
	playing = True
	difficulty = 30


	while playing:
		clock.tick(60) #this slows stuff down from default which is too fast
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				playing = False
                                return
			elif event.type == pygame.MOUSEBUTTONDOWN:
                                if player.mutated and pygame.mouse.get_pressed()[2]:#event.type == pygame.RIGHTMOUSEBUTTONDOWN:
				        newBullett = Torpedo()#Bullett()
                                else:
                                        newBullett = Bullett()
				allSprites.add(newBullett)
				bullettSprites.add(newBullett)
                                bullet_sound.play()
                try:
		        if pygame.time.get_ticks() % difficulty == 0:
		        	newEnemy = Enemy()
		                newEnemy.xmovement = exmove 
		        	allSprites.add(newEnemy)
		        	enemySprites.add(newEnemy)
                except:
                        difficulty = 1


                if score != 0 and  difficulty >= 5 and score % 40 == 0:
                        difficulty -= 4
                        score +=1

                if score != 0 and score % 20 == 0:
                        exmove -= 1
                        score +=1        

		
		for enemy in enemySprites:
			for bullett in bullettSprites:
				if pygame.sprite.collide_rect(enemy, bullett):
					bullett.kill()			
					enemy.register_damage(bullett.damage)
					if enemy.dying:
					        score += 1
                                                scoreLabel.text = str(score)  
                                        spawn_mutagen(enemy.rect, [mutagenSprites, allSprites])
					break

		for enemy in enemySprites:
			if pygame.sprite.collide_rect(player, enemy):
                                enemy.kill()#enemy.register_damage(3)
				player.register_damage(1)
				lifebar.update_damage(player.life)
                                player_damage_sound.play()
				break			

                for mutagen in mutagenSprites:
                        if pygame.sprite.collide_rect(player, mutagen):
                                mutagen.kill()
                                player.mutagen_up()
                                mutagen_up_sound.play()
                                label.text = "Mutagen Up!"
                                if mBarCount < 10:
                                        mBarCount += 1
                                else:
                                        player.mutated = True
                                        mBarCount = 0       
                                mutabar.update_mBar(mBarCount)
                
                if player.life <= 0:
                        quitG =True



                kill_out_of_bounds_sprites()

                allSprites.clear(screen, background) 
        
                if quitG:
                        bSprites.update()
                        bSprites.draw(screen)
                        pygame.display.flip()
                        break

		allSprites.update()
		allSprites.draw(screen)

		pygame.display.flip()
        startG = True               
	while startG:
                for event in pygame.event.get():
	                if event.type == pygame.MOUSEBUTTONDOWN or event.type == pygame.QUIT:
	                        startG = False
	                      
	pygame.mouse.set_visible(True)

	return

        

                                              













